open Base

type t =
  | String of string
  | Stream of (string Lwt_stream.t [@opaque])
[@@deriving show]

let empty =
  String ""

let of_string str =
  String str

let of_stream stream =
  Stream stream

let to_string = function
  | String str -> Some str
  | Stream _ -> None

let to_stream = function
  | String str -> Lwt_stream.of_list [str]
  | Stream stream -> stream

let read_up_to ?(max_bytes=1048576) = function
  | String str ->
    (* Let's not waste time/mem on what's already been read *)
    Lwt.return str
  | Stream stream ->
    let read_bytes = ref 0 in
    let limit_not_reached chunk =
      read_bytes := !read_bytes + String.length chunk;
      !read_bytes < max_bytes in
    Lwt_stream.get_while limit_not_reached stream
    |> Lwt.map String.concat

let length = function
  | String str -> Some (String.length str)
  | Stream _ -> None

let set_encoding_and_length_headers body headers = match length body with
  | Some content_length ->
    headers
    |> Headers.remove ~key:"Transfer-Encoding"
    |> Headers.set ~key:"Content-Length" ~value:(Int.to_string content_length)
  | None ->
    headers
    |> Headers.remove ~key:"Content-Length"
    |> Headers.set ~key:"Transfer-Encoding" ~value:"chunked"
