open Base

type t = {
  client_addr: Unix.sockaddr option [@opaque];
  http_method: string;
  uri: Uri.t;
  headers: Headers.t;
  body: Body.t;
} [@@deriving show]

let get_client_addr {client_addr; _} =
  client_addr

let get_http_method {http_method; _} =
  http_method

let get_uri {uri; _} =
  uri

let get_path {uri; _} =
  Uri.path uri
  |> String.split ~on:'/'
  |> List.filter ~f:(Fn.compose not String.is_empty)

let get_query_string {uri; _} =
  Uri.query uri

let get_headers {headers; _} =
  headers

let get_body {body; _} =
  body

let find_query_params req ~key =
  req
  |> get_query_string
  |> List.find ~f:(function (k, _) -> String.equal k key)
  |> Option.map ~f:snd

let find_query_param req ~key =
  req
  |> find_query_params ~key
  |> Option.bind ~f:List.hd

let find_header req ~key =
  Headers.find req.headers ~key

let make ~http_method ~uri =
  {
    client_addr = None;
    http_method = http_method;
    uri = Uri.of_string uri;
    headers = Headers.empty;
    body = Body.empty;
  }

let get uri =
  make ~http_method:"GET" ~uri

let post uri =
  make ~http_method:"POST" ~uri

let with_body body req =
  let new_headers =
    get_headers req
    |> Body.set_encoding_and_length_headers body in
  { req with body = body; headers = new_headers }

let with_string_body str req =
  with_body (Body.of_string str) req

let with_headers headers req =
  { req with headers = headers }

let with_header ~key ~value req =
  with_headers (Headers.set ~key ~value req.headers) req
