open Alcotest
open Base
open Okweb

let test_create_headers_from_list =
  test_case "create headers from list" `Quick begin fun () ->
    let headers = Headers.of_list [
        "TEST-HEADER", "Test Value 1";
        "TEST-header", "Test Value 2";
        "test-header", "Test Value 3";
      ] in
    check (list (pair string string))
      "headers contain multiple values for the same key"
      ["test-header", "Test Value 3"]
      (Headers.to_list headers)
  end

let test_search_regardless_of_key_case =
  test_case "search regardless of key's case" `Quick begin fun () ->
    let value = "Test Value" in
    let headers = Headers.of_list ["test-header", value] in
    let cases = [
      "test-header";
      "TEST-HEADER";
      "test-HEADER";
      "tEsT-hEaDeR";
    ] in
    List.iter cases ~f:begin fun key ->
      check (option string)
        (Printf.sprintf "couldn't find value for '%s'" key)
        (Some value)
        (Headers.find headers ~key)
    end
  end

let test_remove_header =
  test_case "remove regardless of key's case" `Quick begin fun () ->
    let headers = Headers.of_list [
        "header-1", "value-1";
        "header-2", "value-2";
      ] in
    check (option string) "invalid value" (Some "value-1") (Headers.find headers ~key:"header-1");
    check (option string) "invalid value" (Some "value-2") (Headers.find headers ~key:"header-2");
    let headers = Headers.remove headers ~key:"HEADER-1" in
    check (option string) "invalid value" None (Headers.find headers ~key:"header-1");
    check (option string) "invalid value" (Some "value-2") (Headers.find headers ~key:"header-2");
  end

let cases = [
  test_create_headers_from_list;
  test_search_regardless_of_key_case;
  test_remove_header;
]
